@@ -2,7 +2,7 @@
  *  802.11 WEP replay & injection attacks
  *
  *  Copyright (C) 2006 Thomas d'Otreppe
- *  Copyright (C) 2004,2005  Christophe Devine
+ *  Copyright (C) 2004,2005 Christophe Devine
  *
  *  WEP decryption attack (chopchop) developped by KoreK
  *
@@ -21,21 +21,27 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+
 #include <linux/rtc.h>
-#include <linux/if_ether.h>
-#include <netpacket/packet.h>
-#include <sys/types.h>
+#include <sys/socket.h>
 #include <sys/ioctl.h>
+#include <sys/types.h>
 #include <sys/wait.h>
-#include <sys/stat.h>
 #include <sys/time.h>
+
+#include <netpacket/packet.h>
+#include <linux/if_ether.h>
+#include <linux/wireless.h>
 #include <arpa/inet.h>
-#include <net/if.h>
 #include <unistd.h>
 #include <signal.h>
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <errno.h>
+#include <time.h>
+#include <getopt.h>
+
 #include <fcntl.h>
 #include <errno.h>
 #include <time.h>
@@ -110,19 +116,26 @@ char usage[] =
 "      -h smac   : set Source       MAC address\n"
 "      -e essid  : fakeauth  attack : set target AP SSID\n"
 "      -j        : arpreplay attack : inject FromDS pkts\n"
+"      -g value  : change ring buffer size (default: 8)\n"
+/*
+"\n"
+"  WIDS evasion options:\n"
+"      -y value  : Use packets older than n packets\n"
+"      -z        : Ghosting\n"
+*/
 "\n"
 "  source options:\n"
 "\n"
 "      -i iface  : capture packets from this interface\n"
 "      -r file   : extract packets from this pcap file\n"
 "\n"
-"  attack modes:\n"
+"  attack modes (Numbers can still be used):\n"
 "\n"
-"      --deauth      count : deauthenticate all stations\n"
-"      --fakeauth    delay : fake authentication with AP\n"
-"      --interactive       : interactive frame selection\n"
-"      --arpreplay         : standard ARP-request replay\n"
-"      --chopchop          : decrypt/chopchop WEP packet\n"
+"      --deauth      count : deauthenticate 1 or all stations (-0)\n"
+"      --fakeauth    delay : fake authentication with AP (-1)\n"
+"      --interactive       : interactive frame selection (-2)\n"
+"      --arpreplay         : standard ARP-request replay (-3)\n"
+"      --chopchop          : decrypt/chopchop WEP packet (-4)\n"
 "\n";
 
 struct options
@@ -152,6 +165,9 @@ struct options
     int a_mode;
     int a_count;
     int a_delay;
+    
+    int ringbuffer;
+    int ghost;
 }
 opt;
 
@@ -1234,7 +1250,12 @@ int do_attack_arp_resend( void )
     FILE *f_cap_out;
     struct pcap_file_header pfh_out;
     struct pcap_pkthdr pkh;
-    struct ARP_req arp[8];
+    struct ARP_req * arp;
+
+	if ( opt.ringbuffer )
+		arp = (struct ARP_req*) malloc( opt.ringbuffer * sizeof( struct ARP_req ) );
+	else
+		arp = (struct ARP_req*) malloc( sizeof( struct ARP_req ) );
 
     /* capture only WEP data to broadcast address */
 
@@ -1514,7 +1535,9 @@ add_arp:
 
             nb_arp_tot++;
 
-            if( nb_arp >= 8 )
+			/* Ring buffer size: by default: 8 ) */
+	
+            if( nb_arp >= opt.ringbuffer && opt.ringbuffer > 0)
             {
                 /* no more room, overwrite oldest entry */
 
@@ -1523,11 +1546,9 @@ add_arp:
 
                 if( ++arp_off2 >= nb_arp )
                     arp_off2 = 0;
-            }
-            else
-            {
-                if( ( arp[nb_arp].buf = malloc( 128 ) ) == NULL )
-                {
+            } else {
+            
+                if( ( arp[nb_arp].buf = malloc( 128 ) ) == NULL ) {
                     perror( "malloc failed" );
                     return( 1 );
                 }
@@ -1543,16 +1564,14 @@ add_arp:
 
                 n = sizeof( pkh );
 
-                if( fwrite( &pkh, n, 1, f_cap_out ) != 1 )
-                {
+                if( fwrite( &pkh, n, 1, f_cap_out ) != 1 ) {
                     perror( "fwrite failed" );
                     return( 1 );
                 }
 
                 n = pkh.caplen;
 
-                if( fwrite( h80211, n, 1, f_cap_out ) != 1 )
-                {
+                if( fwrite( h80211, n, 1, f_cap_out ) != 1 ) {
                     perror( "fwrite failed" );
                     return( 1 );
                 }
@@ -2266,9 +2285,10 @@ int main( int argc, char *argv[] )
     opt.f_type    = -1; opt.f_subtype   = -1;
     opt.f_minlen  = -1; opt.f_maxlen    = -1;
     opt.f_tods    = -1; opt.f_fromds    = -1;
-    opt.f_iswep   = -1;
+    opt.f_iswep   = -1; opt.ringbuffer  =  8;
 
     opt.a_mode    = -1; opt.r_fctrl     = -1;
+    opt.ghost     =  0;
 
     while( 1 )
     {
@@ -2284,7 +2304,7 @@ int main( int argc, char *argv[] )
         };
 
         int option = getopt_long( argc, argv,
-                        "b:d:s:m:n:u:v:t:f:w:x:p:a:c:h:e:j:i:r:0:1:234",
+                        "b:d:s:m:n:u:v:t:f:g:w:x:p:a:c:h:e:j:i:r:0:1:234",
                         long_options, &option_index );
 
         if( option < 0 ) break;
@@ -2430,6 +2450,16 @@ int main( int argc, char *argv[] )
                 }
                 break;
 
+			case 'g' :
+
+                if( sscanf( optarg, "%d", &opt.ringbuffer ) != 1 ||
+                    opt.ringbuffer < 1 )
+				{
+					printf( "Invalid replay ring buffer size.\n");
+					return( 1 );
+				}
+				break;
+
             case 'h' :
 
                 if( getmac( optarg, opt.r_smac ) != 0 )
@@ -2470,6 +2500,12 @@ int main( int argc, char *argv[] )
                 opt.s_file = optarg;
                 break;
 
+			case 'z' :
+			
+				opt.ghost = 1;
+			
+				break;
+
             case '0' :
 
                 if( opt.a_mode != -1 )
@@ -2668,7 +2704,7 @@ int main( int argc, char *argv[] )
     {
         memset( strbuf, 0, sizeof( strbuf ) );
         snprintf( strbuf,  sizeof( strbuf ) - 1,
-                  "iwpriv %s rfmontx 1 &>/dev/null",
+                  "iwpriv %s rfmontx 1 >/dev/null 2>/dev/null",
                   argv[optind] );
         system( strbuf );
     }
@@ -2681,7 +2717,7 @@ int main( int argc, char *argv[] )
     	dev.is_madwifi=1;
         memset( strbuf, 0, sizeof( strbuf ) );
         snprintf( strbuf,  sizeof( strbuf ) - 1,
-                  "sysctl -w dev.%s.rawdev=1 &>/dev/null",
+                  "sysctl -w dev.%s.rawdev=1 >/dev/null 2>/dev/null",
                   argv[optind] );
 
         if( system( strbuf ) == 0 )
@@ -2697,15 +2733,13 @@ int main( int argc, char *argv[] )
 #if 0 /* some people reported problems when prismheader is enabled */
             memset( strbuf, 0, sizeof( strbuf ) );
             snprintf( strbuf,  sizeof( strbuf ) - 1,
-                     "sysctl -w dev.%s.rawdev_type=1 &>/dev/null",
+                     "sysctl -w dev.%s.rawdev_type=1 >/dev/null 2>/dev/null",
                      argv[optind] );
             system( strbuf );
 #endif
 
             argv[optind] = athXraw;
-        }
-        else 
-        {
+        } else {
         	// It is madwifi-ng
         	dev.is_madwifing=1;
         }
