@@ -106,6 +106,7 @@ struct AP_info
     int power_lvl[NB_PWR];    /* signal power ring buffer */
     int preamble;             /* 0 = long, 1 = short      */
     int encryption;           /* 0 = none, > 1 = WEP/WPA  */
+	int beacon_logged;        /* We need 1 beacon per AP  */
 
     unsigned long nb_bcn;     /* total number of beacons  */
     unsigned long nb_pkt;     /* total number of packets  */
@@ -167,6 +168,10 @@ struct globals
     struct winsize ws;      /* console window size  */
     
     char * elapsed_time;	/* capture time			*/
+    
+    int one_beacon;         /* Record only 1 beacon?*/
+    
+    int record_data;		/* do we record data?   */
 }
 G;
 
@@ -185,13 +190,14 @@ char usage[] =
 "      --gpsd              : Use GPSd\n"
 "      --write    <prefix> : Dump file prefix (mandatory)\n"
 "      -w                  : same as --write \n"
+"      --beacons           : Record all beacons in dump file\n"
 "\n"
 "  By default, airodump-ng hop on 2.4Ghz channels.\n"
 "  You can make it capture on other/specific channel(s) by using:\n"
 "      --channel <channel> : Capture on a specific channel\n"
 "      -c                  : Same as --channel\n"
 "      -a                  : Will hop on 5Ghz channels\n"
-"      --abg               : Will hop on 2.4Ghz channels\n"
+"      --abg               : Will hop on 2.4 and 5Ghz channels\n"
 "\n";
 
 
@@ -203,6 +209,13 @@ int dump_initialize( char *prefix, int ivs_only )
     FILE *f;
     char ofn[1024];
 
+
+	/* If you only want to see what happening, send all data to /dev/null */
+	
+	if ( prefix == NULL) {
+		return( 0 );	
+	}
+
     /* check not to overflow the ofn buffer */
 
     if( strlen( prefix ) >= sizeof( ofn ) - 10 )
@@ -285,9 +298,8 @@ int dump_initialize( char *prefix, int ivs_only )
             perror( "fwrite(pcap file header) failed" );
             return( 1 );
         }
-    }
-    else
-    {
+    } else {
+
         snprintf( ofn,  sizeof( ofn ) - 1, "%s-%02d.ivs",
                   prefix, G.f_index );
 
@@ -739,6 +751,12 @@ skip_probe:
 
 write_packet:
 
+    if( h80211[0] == 0x80 && G.one_beacon){
+        if( !ap_cur->beacon_logged )
+            ap_cur->beacon_logged = 1;
+        else return ( 0 );
+    }
+
     if( G.f_cap != NULL )
     {
         pkh.caplen = pkh.len = caplen;
@@ -913,21 +931,22 @@ int getBatteryState()
 
     if (linux_apm == 1) 
     {
-        if ((apm = fopen("/proc/apm", "r")) != NULL) {
-            int charging, ac;
-            fgets(buf, 128, apm);
-            fclose(apm);
+        if ((apm = fopen("/proc/apm", "r")) != NULL ) {
+        	if ( fgets(buf, 128,apm) != NULL ) {
+				int charging, ac;
+				fclose(apm);
 
-            sscanf(buf, "%*s %*d.%*d %*x %x %x %x %*d%% %d %s\n", &ac,
-                &charging, &flag, &batteryTime, units);
+				sscanf(buf, "%*s %*d.%*d %*x %x %x %x %*d%% %d %s\n", &ac,
+					&charging, &flag, &batteryTime, units);
 
-            if ((flag & 0x80) == 0 && charging != 0xFF && ac != 1 && batteryTime != -1) {
-                if (!strncmp(units, "min", 32))
-                    batteryTime *= 60;
+				if ((flag & 0x80) == 0 && charging != 0xFF && ac != 1 && batteryTime != -1) {
+					if (!strncmp(units, "min", 32))
+						batteryTime *= 60;
+				}
+				else return 0;
+				linux_acpi = 0;
+				return batteryTime;
             }
-            else return 0;
-            linux_acpi = 0;
-            return batteryTime;
         }
         linux_apm = 0;
     }
@@ -1048,7 +1067,7 @@ int getBatteryState()
     api.ac_state == APM_AC_ON) {
         return 0;
     }
-    return (int)(api.minutes_left);
+    return ((int)(api.minutes_left))*60;
 #else
     return 0;
 #endif
@@ -1348,7 +1367,7 @@ void dump_print( int ws_row, int ws_col )
     }
 }
 
-void dump_write_csv( void )
+int dump_write_csv( void )
 {
     int i, n;
     struct tm *ltime;
@@ -1356,6 +1375,9 @@ void dump_write_csv( void )
     struct AP_info *ap_cur;
     struct ST_info *st_cur;
 
+    if (! G.record_data)
+    	return 0;
+    
     fseek( G.f_txt, 0, SEEK_SET );
 
     fprintf( G.f_txt,
@@ -1494,6 +1516,7 @@ void dump_write_csv( void )
     fprintf( G.f_txt, "\r\n" );
     fflush( G.f_txt );
     sync();
+    return 0;
 }
 
 
@@ -1558,7 +1581,8 @@ void gps_tracker( void )
         if( ( p = strstr( line, "A=" ) ) == NULL ) continue;
         sscanf( p + 2, "%f", &G.gps_loc[4] ); /* altitude */
 
-        fputs( line, G.f_gps );
+        if (G.record_data)
+        	fputs( line, G.f_gps );
 
         G.save_gps = 1;
 
@@ -1778,7 +1802,7 @@ int main( int argc, char *argv[] )
     int n, caplen, pid;
     int fd_raw, arptype;
     int ivs_only, power;
-    int valid_channel;
+    int valid_channel, chanoption;
     time_t tt1, tt2, tt3, start_time;
 
     unsigned char      *buffer;
@@ -1805,6 +1829,7 @@ int main( int argc, char *argv[] )
     memset( &G, 0, sizeof( G ) );
 
     ivs_only       =  0;
+    chanoption     =  0;
     power          = -1;
     fd_raw         = -1;
     arptype        =  0;
@@ -1812,9 +1837,16 @@ int main( int argc, char *argv[] )
     G.batt         =  NULL;
     valid_channel  =  0;
     G.usegpsd      =  0;
-    G.channels     =  abg_chans;
+    G.channels     =  bg_chans;
     G.channel      =  0;
+    G.one_beacon   =  1;
     dump_prefix    =  NULL;
+    G.record_data  =  0;
+	G.f_cap        =  NULL;
+	G.f_ivs        =  NULL;
+	G.f_txt        =  NULL;
+    G.f_gps        =  NULL;
+    
     #ifdef linux
         linux_acpi =  1;
         linux_apm  =  1;
@@ -1822,7 +1854,7 @@ int main( int argc, char *argv[] )
 
     /* check the arguments */
 
-    if( argc < 4 )
+    if( argc < 2 )
     {
         usage:
         printf( usage, getVersion("Airodump-ng", _MAJ, _MIN, _SUB_MIN, _BETA)  );
@@ -1852,6 +1884,7 @@ int main( int argc, char *argv[] )
             {"gba",     0, 0, 'h'},
             {"gab",     0, 0, 'h'},
             {"hop",     0, 0, 'h'},
+            {"beacons", 0, 0, 'e'},
             {0,         0, 0,  0 }
         };
 
@@ -1868,13 +1901,29 @@ int main( int argc, char *argv[] )
                 break;
 
             case 'a' :
-
+				if (chanoption == 1) {
+					printf( "Notice: Channel range already given\n" );
+					break;
+				}
+					
+				chanoption = 1;
                 G.channels = a_chans;
                 break;
 
+			case 'e':
+				G.one_beacon = 0;
+				break;
 
             case 'c' :
 
+				if (G.channel > 0 || chanoption == 1) {
+					if (chanoption == 1)
+						printf( "Notice: Channel range already given\n" );
+					else
+						printf( "Notice: Channel already given (%d)\n", G.channel);
+					break;
+				}
+				
                 sscanf(optarg, "%d",&(G.channel));
                 
                 if ( G.channel < 0 )
@@ -1883,14 +1932,14 @@ int main( int argc, char *argv[] )
                 if (G.channel != 0)
                 {
                     n=0;
-                    do
-                    {
+                    do {
                         if (G.channel == abg_chans[n])
                             break;
-                    }
-                    while (abg_chans[++n]);
+                    } while (abg_chans[++n]);
                     if (G.channel != abg_chans[n])
                         goto usage;
+                    else
+                    	chanoption = 1;
                 }
                 /*
                  * else, hop on 2.4Ghz channels
@@ -1900,11 +1949,20 @@ int main( int argc, char *argv[] )
             case 'g' :              
             case 'b' :
 
+				if (chanoption == 1 && option != 'c') {
+					printf( "Notice: Channel range already given\n" );
+					break;
+				}
+
 				/* Default behaviour: hop on 2.4Ghz channels */
                 G.channels = bg_chans;                      
                 break;
 
             case 'h' :
+				if (chanoption == 1) {
+					printf( "Notice: Channel range already given\n" );
+					break;
+				}
                 
                 /* hop on all available channels */
                 G.channels     =  abg_chans;
@@ -1928,9 +1986,13 @@ int main( int argc, char *argv[] )
                 break;
 
             case 'w':
-            
+            	if (dump_prefix != NULL) {
+            		printf( "Notice: dump prefix already given\n" );
+            		break;
+            	}
                 /* Write prefix */
-                dump_prefix = optarg;
+                dump_prefix   = optarg;
+                G.record_data = 1;
                 break;
 
             default : goto usage;
@@ -1938,11 +2000,10 @@ int main( int argc, char *argv[] )
     }
     while ( 1 );
 
-    if ( dump_prefix == NULL)
-    {
-        printf("Missing dump prefix.\n");
-        return (1);
-    }
+	if ( ivs_only && !G.record_data ) {
+		printf( "Missing dump prefix (-w)\n" );
+		return( 1 );
+	}
 
     /* Check if we have root privileges */
 
@@ -1997,10 +2058,9 @@ int main( int argc, char *argv[] )
 
     /* Exit if ndiswrapper : check iwpriv ndis_reset */
     
-    if ( is_ndiswrapper(iface) )
-    {
-        printf("Ndiswrapper doesn't support monitor mode.\n");
-        return (1);
+    if ( is_ndiswrapper(iface) ) {
+    	printf("Ndiswrapper doesn't support monitor mode.\n");
+    	return (1);
     }
 
     memset( &sll, 0, sizeof( sll ) );
@@ -2056,17 +2116,15 @@ int main( int argc, char *argv[] )
     snprintf( (char*) buffer, strlen( iface ) + 23,
         "/proc/sys/net/%s/%%parent", iface );
     check_madwifing = fopen( (char*) buffer,"r");
-    if (check_madwifing != NULL)
-    {
+    if (check_madwifing != NULL) {
         fclose(check_madwifing);
         G.is_madwifing = 1;
         memset(buffer,0, 65536);
         sprintf((char *) buffer, "/proc/sys/net/%s/dev_type", iface);
-        FILE * f = fopen(buffer,"r");
-        if (f != NULL)
-        {
-            char * madwifiPrism2 = "802";
-            fwrite(madwifiPrism2, 3, 1, f);
+        FILE * f = fopen(buffer,"w");
+        if (f != NULL) {
+            char * madwifiPrism2 = "802\n";
+            fprintf(f, madwifiPrism2);
             fclose(f);
         }
         /* Force prism2 header on madwifi-ng */
@@ -2187,8 +2245,9 @@ int main( int argc, char *argv[] )
 
     /* open or create the output files */
 
-    if( dump_initialize( dump_prefix, ivs_only ) )
-        return( 1 );
+    if (G.record_data)
+    	if( dump_initialize( dump_prefix, ivs_only ) )
+    	    return( 1 );
 
     signal( SIGINT,   sighandler );
     signal( SIGSEGV,  sighandler );
@@ -2383,13 +2442,15 @@ int main( int argc, char *argv[] )
         dump_add_packet( h80211, caplen, power );
     }
 
-    dump_write_csv();
-
-    if( G.f_txt != NULL ) fclose( G.f_txt );
-    if( G.f_gps != NULL ) fclose( G.f_gps );
-    if( G.f_cap != NULL ) fclose( G.f_cap );
-    if( G.f_ivs != NULL ) fclose( G.f_ivs );
+    if (G.record_data) {
+    	dump_write_csv();
 
+    	if( G.f_txt != NULL ) fclose( G.f_txt );
+    	if( G.f_gps != NULL ) fclose( G.f_gps );
+    	if( G.f_cap != NULL ) fclose( G.f_cap );
+    	if( G.f_ivs != NULL ) fclose( G.f_ivs );
+	}
+	
     if( ! G.save_gps )
     {
         sprintf( (char *) buffer, "%s-%02d.gps", argv[2], G.f_index );
